using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using Nash.Multimedia.Riff;

namespace Gvi2Avi
{
    public partial class MainForm : Form
    {
        ProgressForm progress = new ProgressForm();
        ElementRemover converter;

        public MainForm()
        {
            InitializeComponent();
        }

        private void buttonSourceBrowse_Click(object sender, EventArgs e)
        {
            // Create a new Open File Dialog.
            OpenFileDialog openFileDialog = new OpenFileDialog();

            // Set the behaviour of the Open File Dialog.
            openFileDialog.InitialDirectory = textBoxSource.Text;
            openFileDialog.Filter = "Google videos (*.gvi)|*.gvi|All files (*.*)|*.*";
            openFileDialog.FilterIndex = 1;
            openFileDialog.RestoreDirectory = true;


            // Start the Open File Dialog.
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                // Read the Filepath into the textbox.
                textBoxSource.Text = openFileDialog.FileName;
            }
        }

        private void buttonConvert_Click(object sender, EventArgs e)
        {
            // Check if there is a valid file in the Textbox.
            if (File.Exists(textBoxSource.Text))
            {
                // Retrieve information about the file selected.
                FileInfo fi = new FileInfo(textBoxSource.Text);

                // Create a new Save File Dialog.
                SaveFileDialog saveFileDialog = new SaveFileDialog();

                // Set the behaviour of the Save File Dialog.
                saveFileDialog.Filter = "AVI file (*.avi)|*.avi|All files (*.*)|*.*";
                saveFileDialog.FilterIndex = 1;
                saveFileDialog.RestoreDirectory = true;
                saveFileDialog.InitialDirectory = fi.DirectoryName;

                // Default name is the input filename minus the current file extension
                if (fi.Extension.Length == 0)
                {
                    // If the source file has no file extension, it cannot be removed.
                    saveFileDialog.FileName = fi.Name;
                }
                else
                {
                    saveFileDialog.FileName = fi.Name.Remove(fi.Name.Length - fi.Extension.Length);
                }

                // Start the Save File Dialog.
                if (saveFileDialog.ShowDialog() == DialogResult.OK)
                {
                    // Disable the Start button until the asynchronous operation is done.
                    this.buttonConvert.Enabled = false;

                    // Disable the Browse button until the asynchronous operation is done.
                    this.buttonSourceBrowse.Enabled = false;

                    // Disable the Source textbox until the asynchronous operation is done.
                    this.textBoxSource.Enabled = false;

                    String[] args = { textBoxSource.Text, saveFileDialog.FileName };
                    // Do the grunt work of the program.
                    backgroundWorker1.RunWorkerAsync(args);

                    // Display progress.                    
                    progress.Show();
                }
            }
            else
            {
                MessageBox.Show("Please select a file to convert.", "File Not Found", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void buttonQuit_Click(object sender, EventArgs e)
        {
            // Close the Application.            
            Application.Exit();
        }

        private void buttonAbout_Click(object sender, EventArgs e)
        {
            // Display the About box as a modal form.
            AboutBox about = new AboutBox();
            about.ShowDialog(this);
        }

        // This method is the main work performed in the program.
        private void workMethod(String[] files, BackgroundWorker worker, DoWorkEventArgs e)
        {
            // Create a new ElementRemover
            converter = new ElementRemover(files[0], files[1]);

            // Run the conversion.
            String[] list = new String[1];
            list[0] = "goog";
            try
            {
                converter.removeLists(list);
                MessageBox.Show("The file has been converted.", "Finished", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (RiffParserDemo2.RiffParserException ex)
            {
                // Invalid file.
                converter.cleanUp();
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                // Unforseen circumstances.
                converter.cleanUp();
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        // This event handler is where the actual, potentially time-consuming work is done.
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            // Get the BackgroundWorker that raised this event.
            BackgroundWorker worker = sender as BackgroundWorker;

            // Run the method.
            workMethod((String[])e.Argument, worker, e);           
        }

        // This event handler deals with the results of the background operation.
        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            // Stop the progress bar.
            progress.Hide();

            // Enable the Convert button.
            this.buttonConvert.Enabled = true;

            // Enable the Browse button.
            this.buttonSourceBrowse.Enabled = true;

            // Enable the Source textbox.
            this.textBoxSource.Enabled = true;
        }

        // Cleanup any mess when the program is closed.
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Cleanup any work in progress.
            if (converter != null)
            {
                converter.cleanUp();
            }
        }
    }
}